import os
import unittest
from typing import List

from symeo_python.cli.cli import DefaultCliAdapter
from symeo_python.cli.process_runner import ProcessRunnerPort
from symeo_python.configuration.config_loader import SYMEO_LOCAL_FILE, SYMEO_API_KEY
from symeo_python.configuration.config_parser import ConfigParserPort


class CliTest(unittest.TestCase):
    def test_should_parse_conf_from_yaml(self):
        # Given
        conf_parser_mock = ConfigParserAdapterMock()
        process_runner_adapter_mock = ProcessRunnerAdapterMock()
        cli_adapter = DefaultCliAdapter(conf_parser_mock, process_runner_adapter_mock)
        contract_path = "fake/config/contract/path"

        # When
        cli_adapter.generate_configuration_from_contract_file(contract_path)

        # Then
        self.assertEqual(contract_path, conf_parser_mock.configuration_path)
        self.assertIsNone(process_runner_adapter_mock.process)

    def test_should_prepare_env_with_api_key_and_run_sub_process(self):
        # Given
        conf_parser_mock = ConfigParserAdapterMock()
        process_runner_adapter_mock = ProcessRunnerAdapterMock()
        cli_adapter = DefaultCliAdapter(conf_parser_mock, process_runner_adapter_mock)
        api_key = "FAKE_API_KEY_111"
        api_url = "http://fake.111"
        sub_process = ["echo", "'Test1'"]
        cli_input_data = {
            "api_key": api_key,
            "api_url": api_url,
            "sub_process": sub_process,
        }

        # When
        cli_adapter.prepare_env_and_start_sub_process(cli_input_data)

        # Then
        self.assertEqual(sub_process, process_runner_adapter_mock.process)
        self.assertEqual(api_key, os.getenv(SYMEO_API_KEY))

    def test_should_prepare_env_with_local_file_and_run_sub_process(self):
        # Given
        conf_parser_mock = ConfigParserAdapterMock()
        process_runner_adapter_mock = ProcessRunnerAdapterMock()
        cli_adapter = DefaultCliAdapter(conf_parser_mock, process_runner_adapter_mock)
        config_values_path = "./fake_local_file_111.yml"
        api_url = "http://fake.111"
        sub_process = ["echo", "'Test2'"]
        cli_input_data = {
            "config_values_path": config_values_path,
            "api_url": api_url,
            "sub_process": sub_process,
        }

        # When
        cli_adapter.prepare_env_and_start_sub_process(cli_input_data)

        # Then
        self.assertEqual(sub_process, process_runner_adapter_mock.process)
        self.assertEqual(config_values_path, os.getenv(SYMEO_LOCAL_FILE))

    def test_should_raise_exception_for_missing_cli_input_data(self):
        # Given
        conf_parser_mock = ConfigParserAdapterMock()
        process_runner_adapter_mock = ProcessRunnerAdapterMock()
        cli_adapter = DefaultCliAdapter(conf_parser_mock, process_runner_adapter_mock)
        sub_process = ["echo", "'Test2'"]
        cli_input_data = {
            "sub_process": sub_process,
        }

        # When
        exception = None
        try:
            cli_adapter.prepare_env_and_start_sub_process(cli_input_data)
        except Exception as e:
            exception = e

        # Then
        self.assertEqual(
            str(exception), "Missing api_key/api_url or config_values_path"
        )


class ProcessRunnerAdapterMock(ProcessRunnerPort):

    process: str = None

    def run_process(self, process: List[str]) -> None:
        self.process = process


class ConfigParserAdapterMock(ConfigParserPort):
    configuration_path: str

    def generate_configuration(self, configuration_path: str) -> None:
        self.configuration_path = configuration_path



import os

import yaml  # type: ignore

from symeo_python.api_client.symeo_api_client import SymeoApiClientPort
from symeo_python.configuration.configuration import Config

SYMEO_API_KEY = "SYMEO_API_KEY"
SYMEO_API_URL = "SYMEO_API_URL"
SYMEO_LOCAL_FILE = "SYMEO_LOCAL_FILE"


class ConfigLoaderPort:
    def load_config_from_env(self) -> Config:
        pass


class ConfigLoaderAdapter(ConfigLoaderPort):
    def __init__(self, symeo_api_client_port: SymeoApiClientPort):
        self.__symeo_api_client_port = symeo_api_client_port

    __config: Config = None

    def load_config_from_env(self) -> Config:
        if self.__config is not None:
            return self.__config
        if SYMEO_API_KEY in os.environ and SYMEO_API_URL in os.environ:
            return self.__fetch_configuration_values_from_symeo_api()
        elif SYMEO_LOCAL_FILE in os.environ:
            return self.__load_yaml_values()
        else:
            raise Exception(
                "Failed to load Symeo configuration due to missing api key or local configuration values file"
            )

    def __load_yaml_values(self):
        with open(os.getenv(SYMEO_LOCAL_FILE), "r") as yaml_file:
            yaml_data = yaml.load(yaml_file, Loader=yaml.FullLoader)
            config = Config(yaml_data)
            self.__config = config
            return config

    def __fetch_configuration_values_from_symeo_api(self):
        yaml_data = self.__symeo_api_client_port.get_conf_values_for_api_key(
            os.getenv(SYMEO_API_URL), os.getenv(SYMEO_API_KEY)
        )
        config = Config(yaml_data)
        self.__config = config
        return config



import os
from hashlib import md5  # type: ignore
from mmap import ACCESS_READ, mmap  # type: ignore

from symeo_python.yaml_converter.yaml_to_class_converter import YamlToClassPort


class ConfigParserPort:
    def generate_configuration(self, configuration_path: str) -> None:
        pass


class ConfigParserAdapter(ConfigParserPort):

    __SUPPORTED_EXTENSIONS__ = [".yml", ".yaml"]
    __yaml_to_class_port: YamlToClassPort

    def __init__(self, yaml_to_class_port: YamlToClassPort):
        self.__yaml_to_class_port = yaml_to_class_port

    def generate_configuration(self, configuration_path: str):
        if os.path.isfile(configuration_path) is False:
            raise Exception(f"Configuration file {configuration_path} not found")
        else:
            if self.__validate_file_extension(configuration_path):
                self.__yaml_to_class_port.parse_configuration_from_path(
                    configuration_path
                )
            else:
                raise Exception(
                    f"Wrong configuration file format for {configuration_path} . Only .yml and .yaml are accepted"
                )

    def __validate_file_extension(self, configuration_path):
        is_file_extension_supported = False
        for supported_extension in self.__SUPPORTED_EXTENSIONS__:
            if configuration_path.endswith(supported_extension):
                is_file_extension_supported = True
        return is_file_extension_supported

    @staticmethod
    def get_file_md5(file_path) -> str:
        with open(file_path) as file, mmap(  # type: ignore
            file.fileno(), 0, access=ACCESS_READ
        ) as file:
            return md5(file).hexdigest()  # type: ignore



from setuptools import find_packages, setup  # type: ignore

setup(
    name="symeo-python",
    version="0.0.3",
    packages=find_packages(exclude=["test"]),
    install_requires=["pyyaml", "typer[all]", "requests"],
    license="Apache-2.0 license",
    extra_require={},
    url="https://github.com/symeo-io/symeo-python",
    author="Symeo.io",
    author_email="support@symeo.io",
    description="Symeo python SDK with embedded CLI for conf as code",
    entry_points={
        "console_scripts": [
            "symeo-python = symeo_python.cli.main:cli",
        ],
    },
)
